local CM = ChatMonitor
local _G = _G
local PlaySoundFile, UnitIsDead, InCombatLockDown = PlaySoundFile, UnitIsDead, InCombatLockDown
local string, tonumber, ipairs, format = string, tonumber, ipairs, format
local GetPlayerInfoByGUID = GetPlayerInfoByGUID

--GLOBALS: ChatMonitorDB, SLASH_CHATMONITOR_CMD1, SLASH_CHATMONITOR_CMD2, SlashCmdList, ChatMonitorAlertFrame

local function playsound(key)
	local file = CM.LSM:Fetch(CM.LSM.MediaType.SOUND, key)
	PlaySoundFile(file, "Master")
end

local function DoCommands(input)
	if input == "" then
		InterfaceOptionsFrame_OpenToCategory("Chat Monitor")
		InterfaceOptionsFrame_OpenToCategory("Chat Monitor")
	elseif input == "enable" then
		CM.db.char.enabled = true
		CM:CheckChannels()
	elseif input == "disable" then
		CM.db.char.enabled = false
		CM:CheckChannels()
	end
end

function CM:eventhandler(this, event, arg1, ...)
	local amsg
	if event == "ADDON_LOADED" and arg1 == "ChatMonitor" then 
		if ChatMonitorDB == nil then ChatMonitorDB = {} end
		SLASH_CHATMONITOR_CMD1 = "/cm"
		SLASH_CHATMONITOR_CMD2 = "/chatmonitor"
		SlashCmdList["CHATMONITOR_CMD"] = DoCommands
	elseif event == "PLAYER_LOGIN" then
		--configure options
		CM.db = CM.LS("AceDB-3.0"):New("ChatMonitorDB", CM.defaults, "profile")
		--add events and custom sounds
		for _, s in ipairs(CM.CustomSounds) do CM.LSM:Register(CM.LSM.MediaType.SOUND, s.desc, format("Interface\\Addons\\ChatMonitor\\sounds\\%s.ogg", s.name)) end
		CM:CreateOptions()
		CM.alert = CM:CreateAlertFrame().alert
		CM:Skin(ChatMonitorMain)
		if not CM.menu then CM.menu = CreateFrame("Frame", "ChatMonitorMenu", ChatMonitorMain, "UIDropDownMenuTemplate") end
		CM.frame:RegisterEvent("CHANNEL_UI_UPDATE")
		CM.frame:RegisterEvent("ZONE_CHANGED")
		CM.frame:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		CM:CheckChannels()
	elseif event == "CHANNEL_UI_UPDATE" or event == "ZONE_CHANGED" or event == "ZONE_CHANGED_NEW_AREA" then
		CM:CheckChannels()
	elseif event == "CHAT_MSG_CHANNEL" or event == "CHAT_MSG_GUILD" then
		CM:FilterChat(arg1, ...)
	end
end

function CM:PlayAlert(key)
	if CM.db.profile.soundon then
		local file = CM.LSM:Fetch(CM.LSM.MediaType.SOUND, key)
		PlaySoundFile(file, "Master")
	end
end

--function to convert {star}, {square}, etc. to icons
function CM:FormatMessage(msg)
	local term, tag
	for tag in string.gmatch(msg, "%b{}") do
		term = string.lower(string.gsub(tag, "[{}]", ""))
		if _G.ICON_TAG_LIST[term] and _G.ICON_LIST[_G.ICON_TAG_LIST[term]] then
			msg = string.gsub(msg, tag, _G.ICON_LIST[_G.ICON_TAG_LIST[term]] .. "0|t")
		end
	end
	return msg
end

function CM:GetClassColour(guid)
	local _, class, _, _, _ = GetPlayerInfoByGUID(guid)
	local colour = CM.db.profile.playercolour
	local cc = {}

	if class then
		cc = _G.RAID_CLASS_COLORS[class]
		if cc then colour = CM:RGBToHex(cc.r, cc.g, cc.b, 1) end
	end

	return colour
end

function CM:CheckChannels()
	local trade = CM.db.char.chantrade or CM.db.char.changeneral or CM.db.char.chanlfg
	local guild = CM.db.char.changuild

	if CM.db.char.enabled then
		if CM.db.profile.tradeonly then
			local channellist = {GetChannelList()}
			local channels = {}
		
			for chan = 1, #channellist, 2 do
				channels[channellist[chan]] = channellist[chan + 1]
			end
		
			if channels[2] then
				if ChatMonitorMain:GetAlpha() == 0 then UIFrameFadeIn(ChatMonitorMain, 3, 0, 1) end
				if trade then
					if not CM.frame:IsEventRegistered("CHAT_MSG_CHANNEL") then CM.frame:RegisterEvent("CHAT_MSG_CHANNEL") end
				end
				if guild then
					if not CM.frame:IsEventRegistered("CHAT_MSG_GUILD") then CM.frame:RegisterEvent("CHAT_MSG_GUILD") end
				end
			else
				if ChatMonitorMain:GetAlpha() == 1 then UIFrameFadeOut(ChatMonitorMain, 3, 1, 0) end
				if CM.frame:IsEventRegistered("CHAT_MSG_CHANNEL") then CM.frame:UnregisterEvent("CHAT_MSG_CHANNEL") end
				if CM.frame:IsEventRegistered("CHAT_MSG_GUILD") then CM.frame:UnregisterEvent("CHAT_MSG_GUILD") end
			end
		else
			if ChatMonitorMain:GetAlpha() == 0 then UIFrameFadeIn(ChatMonitorMain, 3, 0, 1) end
			if trade then
				if not CM.frame:IsEventRegistered("CHAT_MSG_CHANNEL") then CM.frame:RegisterEvent("CHAT_MSG_CHANNEL") end
			end
			if guild then
				if not CM.frame:IsEventRegistered("CHAT_MSG_GUILD") then CM.frame:RegisterEvent("CHAT_MSG_GUILD") end
			end
		end
	else
		if ChatMonitorMain:GetAlpha() == 1 then UIFrameFadeOut(ChatMonitorMain, 3, 1, 0) end
		if CM.frame:IsEventRegistered("CHAT_MSG_CHANNEL") then CM.frame:UnregisterEvent("CHAT_MSG_CHANNEL") end
		if CM.frame:IsEventRegistered("CHAT_MSG_GUILD") then CM.frame:UnregisterEvent("CHAT_MSG_GUILD") end
	end
end

-- converts e.g. ff00ff00 to r=0, g=1, b=1, a=1
function CM:HexToRGB(hex)
    local ahex, rhex, ghex, bhex = string.sub(hex, 1, 2), string.sub(hex, 3, 4), string.sub(hex, 5, 6), string.sub(hex, 7, 8)
	return tonumber(rhex, 16) / 255, tonumber(ghex, 16) / 255, tonumber(bhex, 16) / 255, tonumber(ahex, 16) / 255
end

-- converts e.g. r=0, g=1, b=0, a=1 to ff00ff00
function CM:RGBToHex(r, g, b, a)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	a = a <= 1 and a >= 0 and a or 0
	return string.format("%02x%02x%02x%02x", a * 255, r * 255, g * 255, b * 255)
end